@@ -15,9 +15,19 @@ from django.shortcuts import render_to_response
 from django.template.context import RequestContext
 from django.utils import six
 from django.utils.encoding import force_text
+from django.utils.http import urlencode
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext_lazy as _
 
+
+try:
+    # Django 1.6 implements this
+    from django.contrib.admin.templatetags.admin_urls import add_preserved_filters
+except ImportError:
+    def add_preserved_filters(context, form_url):
+        return form_url
+
+
 __all__ = (
     'PolymorphicModelChoiceForm', 'PolymorphicParentModelAdmin',
     'PolymorphicChildModelAdmin', 'PolymorphicChildModelFilter'
@@ -38,7 +48,7 @@ class PolymorphicModelChoiceForm(forms.Form):
     The default form for the ``add_type_form``. Can be overwritten and replaced.
     """
     #: Define the label for the radiofield
-    type_label = _("Type")
+    type_label = _('Type')
 
     ct_id = forms.ChoiceField(label=type_label, widget=AdminRadioSelect(attrs={'class': 'radiolist'}))
 
@@ -53,11 +63,11 @@ class PolymorphicChildModelFilter(admin.SimpleListFilter):
     An admin list filter for the PolymorphicParentModelAdmin which enables
     filtering by its child models.
     """
-    title = _('Content type')
+    title = _('Type')
     parameter_name = 'polymorphic_ctype'
 
     def lookups(self, request, model_admin):
-        return model_admin.get_child_type_choices()
+        return model_admin.get_child_type_choices(request, 'change')
 
     def queryset(self, request, queryset):
         try:
@@ -103,6 +113,11 @@ class PolymorphicParentModelAdmin(admin.ModelAdmin):
     add_type_template = None
     add_type_form = PolymorphicModelChoiceForm
 
+    #: The regular expression to filter the primary key in the URL.
+    #: This accepts only numbers as defensive measure against catch-all URLs.
+    #: If your primary key consists of string values, update this regular expression.
+    pk_regex = '(\d+)'
+
 
     def __init__(self, model, admin_site, *args, **kwargs):
         super(PolymorphicParentModelAdmin, self).__init__(model, admin_site, *args, **kwargs)
@@ -163,12 +178,17 @@ class PolymorphicParentModelAdmin(admin.ModelAdmin):
         return self.child_models
 
 
-    def get_child_type_choices(self):
+    def get_child_type_choices(self, request, action):
         """
-        Return a list of polymorphic types which can be added.
+        Return a list of polymorphic types for which the user has the permission to perform the given action.
         """
         choices = []
         for model, _ in self.get_child_models():
+            perm_function_name = 'has_{0}_permission'.format(action)
+            model_admin = self._get_real_admin_by_model(model)
+            perm_function = getattr(model_admin, perm_function_name)
+            if not perm_function(request):
+                continue
             ct = ContentType.objects.get_for_model(model, for_concrete_model=False)
             choices.append((ct.id, model._meta.verbose_name))
         return choices
@@ -206,8 +226,16 @@ class PolymorphicParentModelAdmin(admin.ModelAdmin):
             raise ChildAdminNotRegistered("No child admin site was registered for a '{0}' model.".format(model_class))
 
 
-    def queryset(self, request):
+    def get_queryset(self, request):
         # optimize the list display.
+        qs = super(PolymorphicParentModelAdmin, self).get_queryset(request)
+        if not self.polymorphic_list:
+            qs = qs.non_polymorphic()
+        return qs
+
+
+    # For Django 1.5:
+    def queryset(self, request):
         qs = super(PolymorphicParentModelAdmin, self).queryset(request)
         if not self.polymorphic_list:
             qs = qs.non_polymorphic()
@@ -222,6 +250,12 @@ class PolymorphicParentModelAdmin(admin.ModelAdmin):
             return self.add_type_view(request)
         else:
             real_admin = self._get_real_admin_by_ct(ct_id)
+            # rebuild form_url, otherwise libraries below will override it.
+            form_url = add_preserved_filters({
+                'preserved_filters': urlencode({'ct_id': ct_id}),
+                'opts': self.model._meta},
+                form_url
+            )
             return real_admin.add_view(request, form_url, extra_context)
 
 
@@ -243,11 +277,12 @@ class PolymorphicParentModelAdmin(admin.ModelAdmin):
         Expose the custom URLs for the subclasses and the URL resolver.
         """
         urls = super(PolymorphicParentModelAdmin, self).get_urls()
-        info = self.model._meta.app_label, self.model._meta.module_name
+        meta = self.model._meta
+        info = meta.app_label, getattr(meta, 'model_name', meta.module_name)
 
         # Patch the change URL so it's not a big catch-all; allowing all custom URLs to be added to the end.
         # The url needs to be recreated, patching url.regex is not an option Django 1.4's LocaleRegexProvider changed it.
-        new_change_url = url(r'^(\d+)/$', self.admin_site.admin_view(self.change_view), name='{0}_{1}_change'.format(*info))
+        new_change_url = url(r'^{0}/$'.format(self.pk_regex), self.admin_site.admin_view(self.change_view), name='{0}_{1}_change'.format(*info))
         for i, oldurl in enumerate(urls):
             if oldurl.name == new_change_url.name:
                 urls[i] = new_change_url
@@ -306,7 +341,7 @@ class PolymorphicParentModelAdmin(admin.ModelAdmin):
         if request.META['QUERY_STRING']:
             extra_qs = '&' + request.META['QUERY_STRING']
 
-        choices = self.get_child_type_choices()
+        choices = self.get_child_type_choices(request, 'add')
         if len(choices) == 1:
             return HttpResponseRedirect('?ct_id={0}{1}'.format(choices[0][0], extra_qs))
 
